# © 2019 Le Filament (<http://www.le-filament.com>)
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html).

from datetime import date

from odoo import _, api, fields, models
from odoo.exceptions import ValidationError


class ScopRevision(models.Model):
    _name = "scop.revision"
    _description = "Revision"
    _order = "partner_id, revision_result_year desc, date"

    partner_id = fields.Many2one(
        comodel_name="res.partner",
        string="Organisme",
        domain=[("is_company", "=", True)],
        ondelete="cascade",
        index=True,
    )
    date = fields.Date("Date de révision", index=True)
    delegate_id = fields.Many2one("res.users", string="Réviseur", ondelete="restrict")
    certified_person_id = fields.Many2one(
        comodel_name="res.users",
        string="Réviseur agréé",
        ondelete="restrict",
        tracking=True,
    )
    revision_result_year = fields.Integer("Exercice révisé")
    revision_staff = fields.Integer("Effectif")
    revision_format_id = fields.Many2one(
        "scop.revision.format",
        string="Format de révision",
        ondelete="restrict",
        related="partner_id.revision_format_id",
        store=True,
    )
    revision_type = fields.Selection(
        [
            ("std", "Classique"),
            ("int", "Intermédiaire"),
            ("spc", "Spéciale"),
        ],
        string="Type de révision",
        default="std",
    )
    revision_followup = fields.Selection(
        [
            ("ndef", "Non défini"),
            ("ras", "RAS"),
            ("remarks", "Simples Remarques et Observations"),
            ("reserve", "Réserves"),
            ("corrections", "Mesures Correctives"),
            ("demeure", "Mise en demeure"),
        ],
        string="Suivi après révision",
        default="ndef",
    )
    note = fields.Text("Commentaire")
    facture = fields.Text("Facture")
    facture_reglee = fields.Boolean("Réglée")

    @api.onchange("revision_result_year")
    def _onchange_partner_id(self):
        # Délégué réviseur par défaut
        self.delegate_id = self.partner_id.revision_person_id.id

        # Dans le cas de l'UR idf on calcule une date de révision
        # (pour éviter de saisir la date)
        idf = self.env.ref("cgscop_partner.riga_14231").id
        if self.partner_id.ur_id.id == idf:
            if self.revision_result_year > 2000:
                self.date = date(self.revision_result_year + 1, 1, 1)

    @api.constrains("revision_result_year")
    def _check_revision_result_year(self):
        if (self.revision_result_year > 2040) or (self.revision_result_year < 2010):
            raise ValidationError(_("Exercice révisé invalide"))


class ScopRevisionFormat(models.Model):
    _name = "scop.revision.format"
    _description = "Revision Format"
    _order = "ur_id, name"

    def _default_ur(self):
        return self.env["res.company"]._ur_default_get()

    name = fields.Char("Revision Format")
    ur_id = fields.Many2one(
        "union.regionale",
        string="Union Régionale",
        index=True,
        ondelete="restrict",
        default=_default_ur,
    )
    tarif = fields.Integer("Tarif de vente")
